import pytz
from django import forms
from django.utils.translation import gettext_lazy as _

from web.forms import MultiForm
from web.forms.widgets import CheckboxInput


class PreferencesNotificationsForm(MultiForm):
    email_notification_on_new_comment = forms.BooleanField(
        widget=CheckboxInput, label=_('email notification on new comment'), required=False
    )
    email_notification_on_new_message = forms.BooleanField(
        widget=CheckboxInput, label=_('email notification on new message'), required=False
    )

    def save_notification_settings(self):
        pass


class PreferencesGeneralForm(MultiForm):
    ALL_TIMEZONES = sorted((item, item) for item in pytz.all_timezones)
    tz_name = forms.ChoiceField(choices=ALL_TIMEZONES)



from db.models import Admonition
from web.forms import ModelForm


class AdmonitionForm(ModelForm):
    class Meta:
        model = Admonition
        fields = [
            "subject",
            "content"
        ]



from db.models import AdmonitionTemplate
from web.forms import ModelForm


class AdmonitionTemplateForm(ModelForm):
    class Meta:
        model = AdmonitionTemplate
        fields = [
            "subject",
            "content"
        ]



from django.utils.safestring import mark_safe

from web.forms import Form


class MultiForm(Form):
    id_form = ""

    def __init__(self, id_form: str):
        super().__init__()
        self.id_form = id_form

    def _html_output(self, normal_row, error_row, row_ender, help_text_html, errors_on_separate_row):
        output = super()._html_output(normal_row, error_row, row_ender, help_text_html, errors_on_separate_row)
        output2 = f"<input type='hidden' id='id_form' value='{self.id_form}'>"
        return mark_safe(output + output2)



# Create your tests here.
import unittest
from .bbcode import bbcode2html
from precise_bbcode.bbcode import get_parser

"""
    BBCode tests
"""


class TestBBCodes(unittest.TestCase):
    tests = {
        "email": {
            "source_text": "blabla[email]toto@domain.name[/email]blabla",
            "target_text": "blabla<a href=\"mailto:toto@domain.name\">toto@domain.name</a>blabla",
        },
        "invisible": {
            "source_text": "blabla[invisible]blabla[/invisible]blabla",
            "target_text": "blabla<span style:\"display:none\">blabla</span>blabla",
        },
        "justifier": {
            "source_text": "blabla[justifier]blabla[/justifier]blabla",
            "target_text": "blabla<p style=\"text-align:justify\">blabla</p>blabla",
        },
        "preformatted": {
            "source_text": "blabla[preformatted]blabla[/preformatted]blabla",
            "target_text": "blabla<pre>blabla</pre>blabla",
        },
        "right": {
            "source_text": "blabla[right]blabla[/right]blabla",
            "target_text": "blabla<div align=\"right\">blabla</div>blabla",
        },
        "mp3": {
            "source_text": "blabla[mp3]http://test.com/mp.mp3[/mp3]blabla",
            "target_text": 'blabla<audio controls src="http://test.com/mp.mp3">Your browser does not support the '
                           '<code>audio</code> element.</audio>blabla'
        },
        "br": {
            "source_text": "blabla[br]blabla",
            "target_text": 'blabla<br />blabla'
        },
        "hr": {
            "source_text": "blabla[hr]blabla",
            "target_text": 'blabla<hr />blabla'
        },
    }

    def test_email(self):
        """ test BBCode email """
        bbcode_name = "email"
        re = self.tests.get(bbcode_name)
        result = bbcode2html(re.get("source_text"))
        self.assertEqual(
            first=result,
            second=re.get("target_text"),
            msg=f"Error on rendering tag '{bbcode_name}'"
        )

    def test_invisible(self):
        """ test BBCode invisible """
        bbcode_name = "invisible"
        re = self.tests.get(bbcode_name)
        result = bbcode2html(re.get("source_text"))
        self.assertEqual(
            first=result,
            second=re.get("target_text"),
            msg=f"Error on rendering tag '{bbcode_name}'"
        )

    def test_right(self):
        """ test BBCode right """
        bbcode_name = "right"
        re = self.tests.get(bbcode_name)
        result = bbcode2html(re.get("source_text"))
        self.assertEqual(
            first=result,
            second=re.get("target_text"),
            msg=f"Error on rendering tag '{bbcode_name}'"
        )

    def test_justifier(self):
        """ test BBCode justifier """
        bbcode_name = "justifier"
        re = self.tests.get(bbcode_name)
        result = bbcode2html(re.get("source_text"))
        self.assertEqual(
            first=result,
            second=re.get("target_text"),
            msg=f"Error on rendering tag '{bbcode_name}'"
        )

    def test_preformatted(self):
        """ test BBCode preformatted """
        bbcode_name = "preformatted"
        re = self.tests.get(bbcode_name)
        result = bbcode2html(re.get("source_text"))
        self.assertEqual(
            first=result,
            second=re.get("target_text"),
            msg=f"Error on rendering tag '{bbcode_name}'"
        )

    def test_mp3(self):
        """ test BBCode mp3 """
        bbcode_name = "mp3"
        re = self.tests.get(bbcode_name)
        result = bbcode2html(re.get("source_text"))
        self.assertEqual(
            first=result,
            second=re.get("target_text"),
            msg=f"Error on rendering tag '{bbcode_name}'"
        )

    def test_br(self):
        """ test BBCode br """
        bbcode_name = "br"
        re = self.tests.get(bbcode_name)
        result = bbcode2html(re.get("source_text"))
        self.assertEqual(
            first=result,
            second=re.get("target_text"),
            msg=f"Error on rendering tag '{bbcode_name}'"
        )

    def test_hr(self):
        """ test BBCode hr """
        bbcode_name = "hr"
        re = self.tests.get(bbcode_name)
        result = bbcode2html(re.get("source_text"))
        self.assertEqual(
            first=result,
            second=re.get("target_text"),
            msg=f"Error on rendering tag '{bbcode_name}'"
        )


if __name__ == '__main__':
    unittest.main()



from precise_bbcode.bbcode import get_parser

from precise_bbcode.bbcode.tag import BBCodeTag
from precise_bbcode.tag_pool import tag_pool


class EmailBBCodeTag(BBCodeTag):

    def render(self, value, option=None, parent=None):
        return BBCodeTag.render(value, option, parent)

    name = 'email'
    definition_string = '[email]{EMAIL}[/email]'
    format_string = '<a href="mailto:{EMAIL}">{EMAIL}</a>'

    class Options:
        render_embedded = False
        strip = False
        end_tag_closes = True


tag_pool.register_tag(EmailBBCodeTag)


class InvisibleBBCodeTag(BBCodeTag):

    def render(self, value, option=None, parent=None):
        return BBCodeTag.render(value, option, parent)

    name = 'invisible'
    definition_string = '[invisible]{TEXT}[/invisible]'
    format_string = '<span style:"display:none">{TEXT}</span>'

    class Options:
        render_embedded = False
        strip = False
        end_tag_closes = True


tag_pool.register_tag(InvisibleBBCodeTag)


class RightBBCodeTag(BBCodeTag):

    def render(self, value, option=None, parent=None):
        return BBCodeTag.render(value, option, parent)

    name = 'right'
    definition_string = '[right]{TEXT}[/right]'
    format_string = '<div align="right">{TEXT}</div>'

    class Options:
        render_embedded = False
        strip = False
        end_tag_closes = True


tag_pool.register_tag(RightBBCodeTag)


class JustifierBBCodeTag(BBCodeTag):

    def render(self, value, option=None, parent=None):
        return BBCodeTag.render(value, option, parent)

    name = 'justifier'
    definition_string = '[justifier]{TEXT}[/justifier]'
    format_string = '<p style="text-align:justify">{TEXT}</p>'

    class Options:
        render_embedded = False
        strip = False
        end_tag_closes = True


tag_pool.register_tag(JustifierBBCodeTag)


class PreFormattedBBCodeTag(BBCodeTag):
    def render(self, value, option=None, parent=None):
        return BBCodeTag.render(value, option, parent)

    name = 'preformatted'
    definition_string = '[pre]{TEXT}[/pre]'
    format_string = '<pre>{TEXT}</pre>'

    class Options:
        render_embedded = False
        strip = False
        end_tag_closes = True


tag_pool.register_tag(PreFormattedBBCodeTag)


class Mp3BBCodeTag(BBCodeTag):

    def render(self, value, option=None, parent=None):
        return BBCodeTag.render(value, option, parent)

    name = 'mp3'
    definition_string = '[mp3]{URL}[/mp3]'
    format_string = '<audio controls src=\"{URL}\">Your browser does not support the <code>audio</code> ' \
                    'element.</audio>'

    class Options:
        render_embedded = False
        strip = False
        replace_links = False
        end_tag_closes = True


tag_pool.register_tag(Mp3BBCodeTag)


class BrBBCodeTag(BBCodeTag):

    def render(self, value, option=None, parent=None):
        return BBCodeTag.render(value, option, parent)

    name = 'br'
    definition_string = '[br]'
    format_string = '<br />'

    class Options:
        render_embedded = False
        strip = False
        standalone = True


tag_pool.register_tag(BrBBCodeTag)


class HrBBCodeTag(BBCodeTag):

    def render(self, value, option=None, parent=None):
        return BBCodeTag.render(value, option, parent)

    name = 'hr'
    definition_string = '[hr]'
    format_string = '<hr />'

    class Options:
        render_embedded = False
        strip = False
        standalone = True


tag_pool.register_tag(HrBBCodeTag)


def bbcode2html(text):
    parser = get_parser()
    return parser.render(text)



from glob import glob
import os
from setuptools import setup
from setuptools.command.install import install
from os import getenv, getcwd

# __pkgname__ = "./"

with open('requirements.txt', 'r') as f:
    install_reqs = [
        s for s in [
            line.split('#', 1)[0].strip(' \t\n') for line in f
        ] if s != ''
    ]

# data_files = []
#
# start_point = os.path.join(__pkgname__, 'static')
# for root, dirs, files in os.walk(start_point):
#     root_files = [os.path.join(root, i) for i in files]
#     data_files.append((root, root_files))
#
# start_point = os.path.join(__pkgname__, 'media')
# for root, dirs, files in os.walk(start_point):
#     root_files = [os.path.join(root, i) for i in files]
#     data_files.append((root, root_files))
#
# start_point = os.path.join(__pkgname__, 'doc')
# for root, dirs, files in os.walk(start_point):
#     root_files = [os.path.join(root, i) for i in files]
#     data_files.append((root, root_files))
#
# start_point = os.path.join(__pkgname__, 'templates')
# for root, dirs, files in os.walk(start_point):
#     root_files = [os.path.join(root, i) for i in files]
#     data_files.append((root, root_files))
#
# start_point = os.path.join(__pkgname__, 'logos')
# for root, dirs, files in os.walk(start_point):
#     root_files = [os.path.join(root, i) for i in files]
#     data_files.append((root, root_files))


class InstallWrapper(install):
    media_path = "media"
    static_path = "static"

    def run(self):
        super().run()
        self.static_path = getenv("STATIC_PATH", f"{getcwd()}/{self.static_path}")


setup(
    packages=[
        'api',
        'core',
        'db',
        'editorjs',
        'web',
    ],
    install_requires=install_reqs,
    include_package_data=True,
    setup_requires=['wheel'],
)



# Create your tests here.
import unittest

from db.models import Member


class TestThemesList(unittest.TestCase):
    themes = {
        "THEME_DEFAULT": "muses_default",
        "THEME_CLASSIC": "muses_classic",
        "THEME_ROSE": "muses_rose",
        "THEME_BLUE": "muses_blue",
        "THEME_GREEN": "muses_green",
        "THEME_ORANGE": "muses_orange",
    }

    def test_themes_in_choices(self):
        m = Member()
        for k in self.themes:
            self.assertEqual(self.themes[k], getattr(m, k), msg=f"Member not contains {self.themes[k]}")


